import java.io.*;
import java.io.File;
import java.rmi.Naming;
import java.rmi.RemoteException;
import lejos.remote.ev3.*; 

/**
 * 
 * 
 * 
 * @author Gerhard Rhner
 */
public class JEControl  {
  private static RMIMenu menu;
  private static RemoteEV3 ev3;
  
  /**
  * Command line entry point
  */
  public static void main(String args[]) {
    boolean name = false;
    boolean run = false;
    boolean help = false;
    boolean terminate = false;
    boolean shutdown = false;
    String ev3name = null;
    String filename = null;
    File program = null; 
    
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-n")) {
        name = true;
        if (i + 1 < args.length) {
          ev3name = args[i+1];
          args[i+1] = "";
        } // end of if
      } // end of if
      
      if (args[i].equals("-r")) {
        run = true;
      } // end of if
      
      if (args[i].equals("-h")) {
        help = true;
      } // end of if
      
      if (args[i].equals("-t")) {
        terminate = true;
      } // end of if
      
      if (args[i].equals("-s")) {
        shutdown = true;
      } // end of if
      
    } // end of for
    
    if (help) {
      printHelp();
      System.exit(0);
    }
    
    if (ev3name == null) {
      System.out.println("No hostname specified");
      System.exit(1);
    }  
    
    int lastarg = args.length - 1;
    
    if (lastarg >= 0) {
      filename = args[args.length-1];
      if (!filename.equals("") && (filename.charAt(0) != '-')) {
        program = new File(filename);
        if (!program.exists()) {
          System.out.println("Local file does not exist: " + program);   
          System.exit(1);
        }
      } 
    } 
    
    connect(ev3name);
    
    if (program != null) {
      uploadFile(program);
      
      if (run) {
        runProgram(program);
      } // end of if      
    } // end of if
    
    if (terminate) {
      terminate();
    } // end of if
    
    if (shutdown) {
      shutdown();
    } // end of if    
  }
  
  /**
  * Connect to the EV3
  */
  private static void connect(String name) {
    if (name != null && name.length() > 0) {
      System.out.println("Connecting to " + name);
      try {
        menu = (RMIMenu) Naming.lookup("//" + name + "/RemoteMenu");
        ev3 = new RemoteEV3(name);
        System.out.println("Connected");
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }
  
  /**
  * Upload the specified file
  */
  private static void uploadFile(File file) {
    if (menu == null) return;
    try {
      FileInputStream in = new FileInputStream(file);
      byte[] data = new byte[(int)file.length()];
      in.read(data);
      String remote = "/home/lejos/programs/" + file.getName();
      System.out.println("Uploading: " + file.getPath());
      System.out.println("To: " + remote);
      menu.uploadFile(remote, data);
      in.close();
      System.out.println("Ready");
    } catch (IOException ioe) {
      System.out.println("IOException uploading file");
    }
  }
  
  /**
  * Run the selected program.
  */
  private static void runProgram(File file) {
    if (menu == null) return;
    String program = file.getName().replaceFirst(".jar", "");
    System.out.println("Running: " + program);
    try {
      menu.runProgram(program);
    } catch (RemoteException e) {
      e.printStackTrace();
    }
  }
  
  /**
  * terminate the program on the EV3, if one is running
  */   
  private static void terminate() {
    if (menu == null) return;
    String programName;
    
    try {
      programName = menu.getExecutingProgramName();
    } catch (RemoteException e) {
      System.out.println("Exception getting program name");
      return;
    }
    
    if (programName == null) 
    System.out.println("No executing program");
    else {
      try {
        menu.stopProgram();
        System.out.println("Stopped " + programName);
      } catch (RemoteException e) {
        System.out.println("Exception stopping program");
      }   
    }
  }
  
  /**
  * Shut down the EV3
  */
  private static void shutdown() {
    if (menu == null) 
    return;
    try {
      menu.shutdown();
    } catch (RemoteException e) {
      // Ignore
    }
  }
  
  private static void printHelp() {
    System.out.println("java JEControl [options] [program]");
    System.out.println("-n name  connect to EV3 with this name/ip-adress");
    System.out.println("-r       run [program] after upload");
    System.out.println("-t       terminate the running program");
    System.out.println("-s       shutdown the ev3");
    System.out.println("-h       help");
  }
}
